# vers anterior AML2310.py
import json
import os
import sys

from .request_handler import retry_session, general_call

host = 'https://alchemyml.com' # 127.0.0.1:8011'
url_base = host + '/api'

# api_token = 'eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJ0b2tlbl90eXBlIjoiYWNjZXNzIiwiZXhwIjoxNTgzOTk2Njg1LCJqdGkiOiJiZGUwNzgyOGU5OTI0OGY5YjA0MWU2NGE3OGU2NjU2NyIsInVzZXJfaWQiOjEyfQ.v3KJg0IEWRNUTNnOzaiFX7GZDQjNy1ja7GT0bRQljoQ'

class autentication():
    def get_api_token(self, username, password):
        global api_token # Prescindir de esto!
        url = url_base + '/token/'
        data = json.dumps({'username':username, 'password':password})
        session = retry_session(retries = 10)
        r = session.post(url, data, verify = False)

        if r.status_code == 200:
            tokenJSON = json.loads(r.text)
            api_token = tokenJSON['access']
            return r.status_code, tokenJSON['access']
        else:
            msgJSON = json.loads(r.text)
            msg = msgJSON['message']
            return r.status_code, msg

class dataset(): # datasets? 

    class_name = sys._getframe().f_code.co_name

    def upload(self, *args, **kwargs):  
        str_meth_name = self.class_name + '.' + sys._getframe().f_code.co_name
        input_args = locals()['args']
        input_kwargs = locals()['kwargs']

        return general_call(str_meth_name, input_args, input_kwargs)

    def view(self, *args, **kwargs): # PALABRA RESERVADA!
        str_meth_name = self.class_name + '.' + sys._getframe().f_code.co_name
        input_args = locals()['args']
        input_kwargs = locals()['kwargs']

        return general_call(str_meth_name, input_args, input_kwargs)
    
    def update(self, *args, **kwargs):
        str_meth_name = self.class_name + '.' + sys._getframe().f_code.co_name
        input_args = locals()['args']
        input_kwargs = locals()['kwargs']

        return general_call(str_meth_name, input_args, input_kwargs)

    def delete(self, *args, **kwargs):
        str_meth_name = self.class_name + '.' + sys._getframe().f_code.co_name
        input_args = locals()['args']
        input_kwargs = locals()['kwargs']

        return general_call(str_meth_name, input_args, input_kwargs)

    def statistical_descriptors(self, *args, **kwargs):
        str_meth_name = self.class_name + '.' + sys._getframe().f_code.co_name

        input_args = locals()['args']
        input_kwargs = locals()['kwargs']

        return general_call(str_meth_name, input_args, input_kwargs)

class experiment(): # experiments?

    class_name = sys._getframe().f_code.co_name

    def create(self, *args, **kwargs):
        str_meth_name = self.class_name + '.' + sys._getframe().f_code.co_name
        input_args = locals()['args']
        input_kwargs = locals()['kwargs']

        return general_call(str_meth_name, input_args, input_kwargs)

    def view(self, *args, **kwargs):
        str_meth_name = self.class_name + '.' + sys._getframe().f_code.co_name
        input_args = locals()['args']
        input_kwargs = locals()['kwargs']

        return general_call(str_meth_name, input_args, input_kwargs)

    def update(self, *args, **kwargs):
        str_meth_name = self.class_name + '.' + sys._getframe().f_code.co_name
        input_args = locals()['args']
        input_kwargs = locals()['kwargs']

        return general_call(str_meth_name, input_args, input_kwargs)

    def delete(self, *args, **kwargs):
        str_meth_name = self.class_name + '.' + sys._getframe().f_code.co_name
        input_args = locals()['args']
        input_kwargs = locals()['kwargs']

        return general_call(str_meth_name, input_args, input_kwargs)

    def statistical_descriptors(self, *args, **kwargs):
        str_meth_name = self.class_name + '.' + sys._getframe().f_code.co_name
        input_args = locals()['args']
        input_kwargs = locals()['kwargs']

        return general_call(str_meth_name, input_args, input_kwargs)

    def results(self, *args, **kwargs):
        str_meth_name = self.class_name + '.' + sys._getframe().f_code.co_name
        input_args = locals()['args']
        input_kwargs = locals()['kwargs']

        return general_call(str_meth_name, input_args, input_kwargs)

    def add_to_project(self, *args, **kwargs):
        str_meth_name = self.class_name + '.' + sys._getframe().f_code.co_name
        input_args = locals()['args']
        input_kwargs = locals()['kwargs']

        return general_call(str_meth_name, input_args, input_kwargs)

    def extract_from_project(self, *args, **kwargs):
        str_meth_name = self.class_name + '.' + sys._getframe().f_code.co_name
        input_args = locals()['args']
        input_kwargs = locals()['kwargs']

        return general_call(str_meth_name, input_args, input_kwargs)

    def send(self, *args, **kwargs):
        str_meth_name = self.class_name + '.' + sys._getframe().f_code.co_name
        input_args = locals()['args']
        input_kwargs = locals()['kwargs']

        return general_call(str_meth_name, input_args, input_kwargs)

class project(): # projects?

    class_name = sys._getframe().f_code.co_name

    def create(self, *args, **kwargs):
        str_meth_name = self.class_name + '.' + sys._getframe().f_code.co_name
        input_args = locals()['args']
        input_kwargs = locals()['kwargs']

        return general_call(str_meth_name, input_args, input_kwargs)    

    def view(self, *args, **kwargs):
        str_meth_name = self.class_name + '.' + sys._getframe().f_code.co_name
        input_args = locals()['args']
        input_kwargs = locals()['kwargs']

        return general_call(str_meth_name, input_args, input_kwargs) 

    def update(self, *args, **kwargs):
        str_meth_name = self.class_name + '.' + sys._getframe().f_code.co_name
        input_args = locals()['args']
        input_kwargs = locals()['kwargs']

        return general_call(str_meth_name, input_args, input_kwargs)    

    def delete(self, *args, **kwargs):
        str_meth_name = self.class_name + '.' + sys._getframe().f_code.co_name
        input_args = locals()['args']
        input_kwargs = locals()['kwargs']

        return general_call(str_meth_name, input_args, input_kwargs)    
     



# vers anterior AML2310.py
import json
import os
import requests
import pickle as pkl

from urllib3.util.retry import Retry
from requests.adapters import HTTPAdapter

host = 'https://alchemyml.com' # 127.0.0.1:8011'
url_base = host + '/api'
with open('dict_urlData.pickle', 'rb') as file:
    dict_urlData = pkl.load(file)

def retry_session(retries, session = None, backoff_factor = 0.3, 
                  status_forcelist = (500, 502, 503, 504)):
    session = session or requests.Session()
    retry = Retry(
        total = retries,
        read = retries,
        connect = retries,
        backoff_factor = backoff_factor,
        status_forcelist = status_forcelist,
    )
    adapter = HTTPAdapter(max_retries = retry)
    session.mount('http://', adapter)
    session.mount('https://', adapter)
    return session

def general_call(str_meth_name, input_args, input_kwargs):
    if str_meth_name in dict_urlData.keys():
        if 'token' in input_kwargs:
            api_token = input_kwargs['token']
            input_kwargs.pop('token')
        else:
            api_token = ''

        urlData = url_base + dict_urlData[str_meth_name]
        headers = {'Authorization': 'Bearer ' + api_token}
                
        if input_args:
            input_kwargs['args'] = input_args
            
        mi_data = input_kwargs
        
        if not 'file_path' in input_kwargs.keys():
            headers['Content-type'] = 'application/json'
            session = retry_session(retries = 10)
            api_request = session.post(urlData, headers = headers, json = mi_data, verify = False) # verify quitar

        else:
            file_path = input_kwargs['file_path']
            del input_kwargs['file_path']
            
            if os.path.exists(file_path):
                file_last_modif_date = int(round(os.stat(file_path).st_mtime))
            else:
                reply = {'success': False,
                          'message': 'Result Upload SCRIPT: File not found - NOT valid file path.',
                          'data': {'invalid input': 'File not found - NOT valid file path.'}}
                return ('', reply)
            if str_meth_name == 'dataset.upload':
                mi_data["last_modification_date"] = file_last_modif_date
            files = {'file_path': open(file_path, 'rb')}
            session = retry_session(retries=10)
            api_request = session.post(urlData, headers = headers, 
                                       files = files, data = mi_data, verify = False) # verify quitar
        
        if api_request.status_code == 200:
            res_json = json.loads(api_request.text)
            if 'data' in res_json.keys():
                if isinstance(res_json['data'], dict ) and ('url' in res_json['data'].keys()):
                    session = retry_session(retries=10)
                    r = session.get(res_json['data']['url'], verify = False) # verify quitar
                    f_name = str(res_json['data']['url']).split("/")[-1]
                    open(f_name, 'wb').write(r.content)

                    return api_request.status_code, 'File '+ f_name + ' successfully generated.'
                else:
                    return api_request.status_code, json.loads(api_request.text)
            else:
                return api_request.status_code, json.loads(api_request.text)
    
        else:
            return (api_request.status_code,(json.loads(api_request.text))) 
    